@@ -1043,11 +1043,11 @@ static PmError winmm_write_byte(PmInternal *midi, unsigned char byte,
         m->hdr = hdr = get_free_output_buffer(midi);
         assert(hdr);
         midi->fill_base = (unsigned char *) m->hdr->lpData;
-        midi->fill_offset_ptr = &(hdr->dwBytesRecorded);
+        midi->fill_offset_ptr = (uint32_t *)&(hdr->dwBytesRecorded);
         /* when buffer fills, Pm_WriteSysEx will revert to calling
          * pmwin_write_byte, which expect to have space, so leave
          * one byte free for pmwin_write_byte. Leave another byte
-         * of space for zero after message to make early version of 
+         * of space for zero after message to make early version of
          * MIDI YOKE driver happy -- therefore dwBufferLength - 2 */
         midi->fill_length = hdr->dwBufferLength - 2;
         if (midi->latency != 0) {
