@@ -138,19 +138,14 @@ void sighip_setup(void)
 
 /* ---------------- lop~ - 1-pole lopass filter. ----------------- */
 
-typedef struct lopctl
-{
-    t_sample c_x;
-    t_sample c_coef;
-} t_lopctl;
-
 typedef struct siglop
 {
     t_object x_obj;
-    t_float x_sr;
-    t_float x_hz;
-    t_lopctl x_cspace;
-    t_float x_f;
+    t_float x_conversion;   /* frequency-to-coefficient conversion factor */
+    t_sample x_last;        /* last output */
+    t_float x_hz;           /* rolloff frequency in hz  (for scalar inlet) */
+    t_sample x_coef;        /* filter coefficient (for scalar inlet) */
+    t_float x_f;            /* value of first inlet if unconnected */
 } t_siglop;
 
 t_class *siglop_class;
@@ -160,67 +155,87 @@ static void siglop_ft1(t_siglop *x, t_floatarg f);
 static void *siglop_new(t_floatarg f)
 {
     t_siglop *x = (t_siglop *)pd_new(siglop_class);
-    inlet_new(&x->x_obj, &x->x_obj.ob_pd, gensym("float"), gensym("ft1"));
+    signalinlet_new(&x->x_obj, f);
     outlet_new(&x->x_obj, &s_signal);
-    x->x_sr = 44100;
-    x->x_cspace.c_x = 0;
-    siglop_ft1(x, f);
+    x->x_conversion =  x->x_last = x->x_hz = x->x_coef = 0;
     x->x_f = 0;
     return (x);
 }
 
-static void siglop_ft1(t_siglop *x, t_floatarg f)
-{
-    if (f < 0) f = 0;
-    x->x_hz = f;
-    x->x_cspace.c_coef = f * (2 * 3.14159) / x->x_sr;
-    if (x->x_cspace.c_coef > 1)
-        x->x_cspace.c_coef = 1;
-    else if (x->x_cspace.c_coef < 0)
-        x->x_cspace.c_coef = 0;
-}
-
 static void siglop_clear(t_siglop *x, t_floatarg q)
 {
-    x->x_cspace.c_x = 0;
+    x->x_last = 0;
 }
 
-static t_int *siglop_perform(t_int *w)
+static t_int *siglop_perf_scalar(t_int *w)
 {
-    t_sample *in = (t_sample *)(w[1]);
-    t_sample *out = (t_sample *)(w[2]);
-    t_lopctl *c = (t_lopctl *)(w[3]);
-    int n = (int)w[4];
-    int i;
-    t_sample last = c->c_x;
-    t_sample coef = c->c_coef;
-    t_sample feedback = 1 - coef;
+    t_siglop *x = (t_siglop *)(w[1]);
+    t_sample *in1 = (t_sample *)(w[2]);
+    t_sample newhz = *(t_sample *)(w[3]);
+    t_sample *out = (t_sample *)(w[4]);
+    int i, n = (int)w[5];
+    t_sample last = x->x_last, coef, feedback;
+    if (newhz != x->x_hz)
+    {
+        x->x_hz = newhz;
+        coef = newhz * x->x_conversion;
+        if (coef > 1)
+            coef = 1;
+        else if (coef < 0)
+            coef = 0;
+        x->x_coef = coef;
+    }
+    else coef = x->x_coef;
+    feedback = 1.f - coef;
     for (i = 0; i < n; i++)
-        last = *out++ = coef * *in++ + feedback * last;
+        last = *out++ = coef * *in1++ + feedback * last;
     if (PD_BIGORSMALL(last))
         last = 0;
-    c->c_x = last;
-    return (w+5);
+    x->x_last = last;
+    return (w+6);
+}
+
+static t_int *siglop_perf_vector(t_int *w)
+{
+    t_siglop *x = (t_siglop *)(w[1]);
+    t_sample *in1 = (t_sample *)(w[2]);
+    t_sample *in2 = (t_sample *)(w[3]);
+    t_sample *out = (t_sample *)(w[4]);
+    int i, n = (int)w[5];
+    t_sample last = x->x_last, coef;
+
+    for (i = 0; i < n; i++)
+    {
+        coef = *in2++ * x->x_conversion;
+        if (coef > 1)
+            coef = 1;
+        else if (coef < 0)
+            coef = 0;
+        last = *out++ = coef * *in1++ + (1.f - coef) * last;
+        /* this formulation seems to run slower, at least on Intel hardware:
+            *out++ = (last += coef * (*in1++ - last)); */
+    }
+    if (PD_BIGORSMALL(last))
+        last = 0;
+    x->x_last = last;
+    return (w+6);
 }
 
 static void siglop_dsp(t_siglop *x, t_signal **sp)
 {
-    x->x_sr = sp[0]->s_sr;
-    siglop_ft1(x,  x->x_hz);
-    dsp_add(siglop_perform, 4,
-        sp[0]->s_vec, sp[1]->s_vec,
-            &x->x_cspace, (t_int)sp[0]->s_n);
+    x->x_conversion = (2*3.14159)/sp[0]->s_sr;
+    x->x_hz = x->x_coef = 0;    /* this will be updated at perf time */
+    dsp_add((sp[1]->s_n > 1 ? siglop_perf_vector : siglop_perf_scalar), 5,
+        x, sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
 }
 
 void siglop_setup(void)
 {
     siglop_class = class_new(gensym("lop~"), (t_newmethod)siglop_new, 0,
-        sizeof(t_siglop), 0, A_DEFFLOAT, 0);
+        sizeof(t_siglop), CLASS_NOPROMOTESIG, A_DEFFLOAT, 0);
     CLASS_MAINSIGNALIN(siglop_class, t_siglop, x_f);
     class_addmethod(siglop_class, (t_method)siglop_dsp,
         gensym("dsp"), A_CANT, 0);
-    class_addmethod(siglop_class, (t_method)siglop_ft1,
-        gensym("ft1"), A_FLOAT, 0);
     class_addmethod(siglop_class, (t_method)siglop_clear, gensym("clear"), 0);
 }
 
