@@ -123,7 +123,7 @@ static void class_addmethodtolist(t_class *c, t_methodentry **methodlist,
         if (sel && (*methodlist)[i].me_name == sel)
     {
         char nbuf[80];
-        snprintf(nbuf, 80, "%s_aliased", sel->s_name);
+        pd_snprintf(nbuf, 80, "%s_aliased", sel->s_name);
         nbuf[79] = 0;
         (*methodlist)[i].me_name = dogensym(nbuf, 0, pdinstance);
         if (c == pd_objectmaker)
@@ -142,11 +142,16 @@ static void class_addmethodtolist(t_class *c, t_methodentry **methodlist,
 }
 
 #ifdef PDINSTANCE
-EXTERN void pd_setinstance(t_pdinstance *x)
+void pd_setinstance(t_pdinstance *x)
 {
     pd_this = x;
 }
 
+t_pdinstance *pd_getinstance(void)
+{
+    return pd_this;
+}
+
 static void pdinstance_renumber(void)
 {
     int i;
@@ -157,7 +162,7 @@ static void pdinstance_renumber(void)
 extern void text_template_init(void);
 extern void garray_init(void);
 
-EXTERN t_pdinstance *pdinstance_new(void)
+t_pdinstance *pdinstance_new(void)
 {
     t_pdinstance *x = (t_pdinstance *)getbytes(sizeof(t_pdinstance));
     t_class *c;
@@ -193,7 +198,7 @@ EXTERN t_pdinstance *pdinstance_new(void)
     return (x);
 }
 
-EXTERN void pdinstance_free(t_pdinstance *x)
+void pdinstance_free(t_pdinstance *x)
 {
     t_symbol *s;
     t_canvas *canvas;
@@ -203,7 +208,7 @@ EXTERN void pdinstance_free(t_pdinstance *x)
     pd_setinstance(x);
     sys_lock();
     pd_globallock();
-    
+
     instanceno = x->pd_instanceno;
     inter = x->pd_inter;
     canvas_suspend_dsp();
@@ -569,28 +574,25 @@ void class_addcreator(t_newmethod newmethod, t_symbol *s,
     t_atomtype type1, ...)
 {
     va_list ap;
-    t_atomtype vec[MAXPDARG+1], *vp = vec;
+    t_atomtype vec[MAXPDARG+1], *vp = vec, argtype = type1;
     int count = 0;
-    *vp = type1;
 
     va_start(ap, type1);
-    while (*vp)
-    {
-        if (count == MAXPDARG)
-        {
-            if(s)
-                pd_error(0, "class %s: sorry: only %d creation args allowed",
-                      s->s_name, MAXPDARG);
-            else
-                pd_error(0, "unnamed class: sorry: only %d creation args allowed",
-                      MAXPDARG);
-            break;
-        }
-        vp++;
-        count++;
-        *vp = va_arg(ap, t_atomtype);
+    while(argtype != A_NULL && count < MAXPDARG) {
+        vec[count++] = argtype;
+        argtype = va_arg(ap, t_atomtype);
     }
     va_end(ap);
+        /* the last argument must be A_NULL */
+    if (A_NULL != argtype) {
+        if(s)
+            pd_error(0, "class %s: sorry: only %d creation args allowed",
+                s->s_name, MAXPDARG);
+        else
+            pd_error(0, "unnamed class: sorry: only %d creation args allowed",
+                MAXPDARG);
+    }
+    vec[count] = A_NULL;
     class_addmethod(pd_objectmaker, (t_method)newmethod, s,
         vec[0], vec[1], vec[2], vec[3], vec[4], vec[5]);
 }
@@ -937,23 +939,22 @@ t_pd *pd_newest(void)
 
     /* horribly, we need prototypes for each of the artificial function
     calls in typedmess(), to keep the compiler quiet. */
+    /* Even more horribly, function pointers must be called at their
+    exact type in Emscripten, including return type, which makes the
+    number of cases explode. The required typedefs and dispatcher code
+    are generated by m_class_dispatcher.c, which see. */
+
 typedef t_pd *(*t_newgimme)(t_symbol *s, int argc, t_atom *argv);
 typedef void(*t_messgimme)(t_pd *x, t_symbol *s, int argc, t_atom *argv);
+typedef void*(*t_messgimmer)(t_pd *x, t_symbol *s, int argc, t_atom *argv);
 
-typedef t_pd *(*t_fun0)(
-    t_floatarg d1, t_floatarg d2, t_floatarg d3, t_floatarg d4, t_floatarg d5);
-typedef t_pd *(*t_fun1)(t_int i1,
-    t_floatarg d1, t_floatarg d2, t_floatarg d3, t_floatarg d4, t_floatarg d5);
-typedef t_pd *(*t_fun2)(t_int i1, t_int i2,
-    t_floatarg d1, t_floatarg d2, t_floatarg d3, t_floatarg d4, t_floatarg d5);
-typedef t_pd *(*t_fun3)(t_int i1, t_int i2, t_int i3,
-    t_floatarg d1, t_floatarg d2, t_floatarg d3, t_floatarg d4, t_floatarg d5);
-typedef t_pd *(*t_fun4)(t_int i1, t_int i2, t_int i3, t_int i4,
-    t_floatarg d1, t_floatarg d2, t_floatarg d3, t_floatarg d4, t_floatarg d5);
-typedef t_pd *(*t_fun5)(t_int i1, t_int i2, t_int i3, t_int i4, t_int i5,
-    t_floatarg d1, t_floatarg d2, t_floatarg d3, t_floatarg d4, t_floatarg d5);
-typedef t_pd *(*t_fun6)(t_int i1, t_int i2, t_int i3, t_int i4, t_int i5, t_int i6,
-    t_floatarg d1, t_floatarg d2, t_floatarg d3, t_floatarg d4, t_floatarg d5);
+    /* this file is generated by m_dispatch_gen.c */
+#include "m_dispatch.h"
+
+void *bang_new(t_pd *dummy);
+void *pdfloat_new(t_pd *dummy, t_float f);
+void *pdsymbol_new(t_pd *dummy, t_symbol *s);
+void *list_new(t_pd *dummy, t_symbol *s, int argc, t_atom *argv);
 
 void pd_typedmess(t_pd *x, t_symbol *s, int argc, t_atom *argv)
 {
@@ -964,35 +965,59 @@ void pd_typedmess(t_pd *x, t_symbol *s, int argc, t_atom *argv)
     int i;
     t_int ai[MAXPDARG+1], *ap = ai;
     t_floatarg ad[MAXPDARG+1], *dp = ad;
-    int narg = 0;
-    t_pd *bonzo;
+    int niarg = 0;
+    int nfarg = 0;
 
         /* check for messages that are handled by fixed slots in the class
         structure. */
     if (s == &s_float)
     {
-        if (!argc) (*c->c_floatmethod)(x, 0.);
-        else if (argv->a_type == A_FLOAT)
-            (*c->c_floatmethod)(x, argv->a_w.w_float);
-        else goto badarg;
+        if (x == &pd_objectmaker)
+        {
+            if (!argc)
+                pd_this->pd_newest = pdfloat_new(x, 0.);
+            else if (argv->a_type == A_FLOAT)
+                pd_this->pd_newest = pdfloat_new(x, argv->a_w.w_float);
+            else goto badarg;
+        }
+        else
+        {
+            if (!argc)
+                (*c->c_floatmethod)(x, 0.);
+            else if (argv->a_type == A_FLOAT)
+                (*c->c_floatmethod)(x, argv->a_w.w_float);
+            else goto badarg;
+        }
         return;
     }
     if (s == &s_bang)
     {
-        (*c->c_bangmethod)(x);
+        if (x == &pd_objectmaker)
+            pd_this->pd_newest = bang_new(x);
+        else
+            (*c->c_bangmethod)(x);
         return;
     }
     if (s == &s_list)
     {
-        (*c->c_listmethod)(x, s, argc, argv);
+        if (x == &pd_objectmaker)
+            pd_this->pd_newest = list_new(x, s, argc, argv);
+        else
+            (*c->c_listmethod)(x, s, argc, argv);
         return;
     }
     if (s == &s_symbol)
     {
         if (argc && argv->a_type == A_SYMBOL)
-            (*c->c_symbolmethod)(x, argv->a_w.w_symbol);
+           if (x == &pd_objectmaker)
+                pd_this->pd_newest = pdsymbol_new(x, argv->a_w.w_symbol);
+           else
+                (*c->c_symbolmethod)(x, argv->a_w.w_symbol);
         else
-            (*c->c_symbolmethod)(x, &s_);
+           if (x == &pd_objectmaker)
+                pd_this->pd_newest = pdsymbol_new(x, &s_);
+           else
+                (*c->c_symbolmethod)(x, &s_);
         return;
     }
         /* pd_objectmaker doesn't require
@@ -1018,11 +1043,14 @@ void pd_typedmess(t_pd *x, t_symbol *s, int argc, t_atom *argv)
             if (x == &pd_objectmaker)
                 pd_this->pd_newest =
                     (*((t_newgimme)(m->me_fun)))(s, argc, argv);
-            else (*((t_messgimme)(m->me_fun)))(x, s, argc, argv);
+            else if (((t_messgimmer)(m->me_fun)) == ((t_messgimmer)(canvas_new)))
+                (*((t_messgimmer)(m->me_fun)))(x, s, argc, argv);
+            else
+                (*((t_messgimme)(m->me_fun)))(x, s, argc, argv);
             return;
         }
         if (argc > MAXPDARG) argc = MAXPDARG;
-        if (x != &pd_objectmaker) *(ap++) = (t_int)x, narg++;
+        if (x != &pd_objectmaker) *(ap++) = (t_int)x, niarg++;
         while ((wanttype = *wp++))
         {
             switch (wanttype)
@@ -1037,7 +1065,7 @@ void pd_typedmess(t_pd *x, t_symbol *s, int argc, t_atom *argv)
                     argc--;
                     argv++;
                 }
-                narg++;
+                niarg++;
                 ap++;
                 break;
             case A_FLOAT:
@@ -1052,6 +1080,7 @@ void pd_typedmess(t_pd *x, t_symbol *s, int argc, t_atom *argv)
                     argc--;
                     argv++;
                 }
+                nfarg++;
                 dp++;
                 break;
             case A_SYMBOL:
@@ -1073,36 +1102,15 @@ void pd_typedmess(t_pd *x, t_symbol *s, int argc, t_atom *argv)
                     argc--;
                     argv++;
                 }
-                narg++;
+                niarg++;
                 ap++;
                 break;
             default:
                 goto badarg;
             }
         }
-        switch (narg)
-        {
-        case 0 : bonzo = (*(t_fun0)(m->me_fun))
-            (ad[0], ad[1], ad[2], ad[3], ad[4]); break;
-        case 1 : bonzo = (*(t_fun1)(m->me_fun))
-            (ai[0], ad[0], ad[1], ad[2], ad[3], ad[4]); break;
-        case 2 : bonzo = (*(t_fun2)(m->me_fun))
-            (ai[0], ai[1], ad[0], ad[1], ad[2], ad[3], ad[4]); break;
-        case 3 : bonzo = (*(t_fun3)(m->me_fun))
-            (ai[0], ai[1], ai[2], ad[0], ad[1], ad[2], ad[3], ad[4]); break;
-        case 4 : bonzo = (*(t_fun4)(m->me_fun))
-            (ai[0], ai[1], ai[2], ai[3],
-                ad[0], ad[1], ad[2], ad[3], ad[4]); break;
-        case 5 : bonzo = (*(t_fun5)(m->me_fun))
-            (ai[0], ai[1], ai[2], ai[3], ai[4],
-                ad[0], ad[1], ad[2], ad[3], ad[4]); break;
-        case 6 : bonzo = (*(t_fun6)(m->me_fun))
-            (ai[0], ai[1], ai[2], ai[3], ai[4], ai[5],
-                ad[0], ad[1], ad[2], ad[3], ad[4]); break;
-        default: bonzo = 0;
-        }
-        if (x == &pd_objectmaker)
-            pd_this->pd_newest = bonzo;
+            /* mess_dispatch() is defined in m_dispatch.h */
+        mess_dispatch(x, m->me_fun, niarg, ai, nfarg, ad);
         return;
     }
     (*c->c_anymethod)(x, s, argc, argv);
@@ -1240,7 +1248,7 @@ t_class *
         logpost(0, loglevel, "refusing to load %dbit-float object '%s' into %dbit-float Pd", ext_floatsize, s->s_name, PD_FLOATSIZE);
         loglevel=3;
     } else
-        logpost(0, 3, "refusing to load unnamed %dbit-float object into %dbit-float Pd", ext_floatsize, PD_FLOATSIZE);
+        logpost(0, PD_DEBUG, "refusing to load unnamed %dbit-float object into %dbit-float Pd", ext_floatsize, PD_FLOATSIZE);
 
     return 0;
 }
@@ -1252,4 +1260,3 @@ int class_getdspflags(const t_class *c)
             (c->c_nopromotesig ? CLASS_NOPROMOTESIG : 0) |
             (c->c_nopromoteleft ? CLASS_NOPROMOTELEFT : 0) );
 }
-
