@@ -6,19 +6,13 @@
     the main way in for Mac OS and, with Michael Casey's help, also into
     ASIO in Windows.
 
-    Both blocking and non-blocking call styles are supported.  If non-blocking
-    is requested, either we call portaudio in non-blocking mode, or else we
-    call portaudio in callback mode and manage our own FIFO se we can offer
-    Pd "blocking" I/O calls.  To do the latter we define FAKEBLOCKING; this
-    works better in MACOSX (gets 40 msec lower latency!) and might also in
-    Windows.  If FAKEBLOCKING is defined we can choose between two methods
-    for waiting on the (presumably other-thread) I/O to complete, either
-    correct thread synchronization (by defining THREADSIGNAL) or just sleeping
-    and polling; the latter seems to work better so far.
-*/
+    For the polling scheduler, we call portaudio in callback mode and manage
+    our own FIFO, so we can offer Pd "blocking" I/O calls. We can choose
+    between two methods for waiting on the I/O to complete, either correct
+    thread synchronization (by defining THREADSIGNAL) or just sleeping and polling.
 
-/* dolist...
-    switch to usleep in s_inter.c
+    For the callback scheduler, we call portaudio in callback mode and simply
+    provide a callback function that ticks the scheduler, polls the GUI, etc.
 */
 
 #include "m_pd.h"
@@ -34,18 +28,17 @@
 #endif
 
 #include "m_private_utils.h"
+#include "s_audio_paring.h"
 
 #ifndef _WIN32          /* for the "dup2" workaround -- do we still need it? */
 #include <unistd.h>
 #endif
 
+/* enable proper thread synchronization instead of polling */
 #if 1
-#define FAKEBLOCKING
+#define THREADSIGNAL
 #endif
 
-/* define this to enable thread signaling instead of polling */
-// #define THREADSIGNAL
-
     /* LATER try to figure out how to handle default devices in portaudio;
     the way s_audio.c handles them isn't going to work here. */
 
@@ -58,39 +51,21 @@ static t_sample *pa_soundin, *pa_soundout;
 static t_audiocallback pa_callback;
 
 static int pa_started;
-static int pa_nbuffers;
-static int pa_dio_error;
+static volatile int pa_dio_error;
 
-#ifdef FAKEBLOCKING
-#include "s_audio_paring.h"
-static PA_VOLATILE char *pa_outbuf;
-static PA_VOLATILE sys_ringbuf pa_outring;
-static PA_VOLATILE char *pa_inbuf;
-static PA_VOLATILE sys_ringbuf pa_inring;
+static char *pa_outbuf;
+static sys_ringbuf pa_outring;
+static char *pa_inbuf;
+static sys_ringbuf pa_inring;
 #ifdef THREADSIGNAL
-#include <pthread.h>
-#include <errno.h>
-#ifdef _WIN32
-#include <sys/timeb.h>
-#else
-#include <sys/time.h>
+t_semaphore *pa_sem;
 #endif
-/* maximum time (in ms) to wait for the condition variable. */
-#ifndef THREADSIGNAL_TIMEOUT
-#define THREADSIGNAL_TIMEOUT 1000
-#endif
-static pthread_mutex_t pa_mutex;
-static pthread_cond_t pa_sem;
-#else /* THREADSIGNAL */
-#if defined (FAKEBLOCKING) && defined(_WIN32)
-#include <windows.h>    /* for Sleep() */
-#endif
-/* max time (in ms) before trying to reopen the device */
+/* number of seconds to wait before checking the audio device
+(and possibly trying to reopen it). */
 #ifndef POLL_TIMEOUT
-#define POLL_TIMEOUT 2000
+#define POLL_TIMEOUT 2.0
 #endif
-#endif /* THREADSIGNAL */
-#endif  /* FAKEBLOCKING */
+static double pa_lastdactime;
 
 static void pa_init(void)        /* Initialize PortAudio  */
 {
@@ -118,7 +93,7 @@ static void pa_init(void)        /* Initialize PortAudio  */
 #endif
 
 
-        if ( err != paNoError )
+        if (err != paNoError)
         {
             post("error#%d opening audio: %s", err, Pa_GetErrorText(err));
             return;
@@ -171,15 +146,16 @@ static int pa_lowlevel_callback(const void *inputBuffer,
     return 0;
 }
 
-#ifdef FAKEBLOCKING
     /* callback for "non-callback" case in which we actually open portaudio
-    in callback mode but fake "blocking mode". We communicate with the
-    main thread via FIFO.  First read the audio output FIFO (which
-    we sync on, not waiting for it but supplying zeros to the audio output if
-    there aren't enough samples in the FIFO when we are called), then write
-    to the audio input FIFO.  The main thread will wait for the input fifo.
-    We can either throw it a pthreads condition or just allow the main thread
-    to poll for us; so far polling seems to work better. */
+    in callback mode but fake "blocking mode". We communicate with the main
+    thread via two FIFOs: the audio input buffer is copied to our input FIFO
+    and the audio output buffer is filled with samples from our output FIFO.
+    If the input FIFO is full or the output FIFO is empty, the callback does
+    not wait for the scheduler; instead, the input buffer is discarded and
+    the output buffer is filled with zeros. After each scheduler tick, the
+    main thread tries to drain the input FIFO and fill the output FIFO.
+    It can either wait on a semaphore, or poll in regular intervals;
+    the first option should be more responsive. */
 static int pa_fifo_callback(const void *inputBuffer,
     void *outputBuffer, unsigned long nframes,
     const PaStreamCallbackTimeInfo *outTime, PaStreamCallbackFlags myflags,
@@ -187,19 +163,35 @@ static int pa_fifo_callback(const void *inputBuffer,
 {
     /* callback routine for non-callback client... throw samples into
             and read them out of a FIFO */
+    unsigned long infiforoom = sys_ringbuf_getwriteavailable(&pa_inring),
+        outfiforoom = sys_ringbuf_getreadavailable(&pa_outring);
     int ch;
-    long fiforoom;
-    float *fbuf;
 
 #if CHECKFIFOS
     if (pa_inchans * sys_ringbuf_getreadavailable(&pa_outring) !=
         pa_outchans * sys_ringbuf_getwriteavailable(&pa_inring))
             post("warning: in and out rings unequal (%d, %d)",
                 sys_ringbuf_getreadavailable(&pa_outring),
-                 sys_ringbuf_getwriteavailable(&pa_inring));
+                sys_ringbuf_getwriteavailable(&pa_inring));
 #endif
-    fiforoom = sys_ringbuf_getreadavailable(&pa_outring);
-    if ((unsigned)fiforoom >= nframes*pa_outchans*sizeof(float))
+    if (infiforoom < nframes * STUFF->st_inchannels * sizeof(t_sample) ||
+        outfiforoom < nframes * STUFF->st_outchannels * sizeof(t_sample))
+    {
+        /* data late: output zeros, drop inputs, and leave FIFos untouched */
+        if (pa_started)
+            pa_dio_error = 1;
+        if (outputBuffer)
+        {
+            for (ch = 0; ch < pa_outchans; ch++)
+            {
+                unsigned long frame;
+                float *fbuf = ((float *)outputBuffer) + ch;
+                for (frame = 0; frame < nframes; frame++, fbuf += pa_outchans)
+                    *fbuf = 0;
+            }
+        }
+    }
+    else
     {
         if (outputBuffer)
             sys_ringbuf_read(&pa_outring, outputBuffer,
@@ -212,34 +204,16 @@ static int pa_fifo_callback(const void *inputBuffer,
         else if (pa_inchans)
             post("audio error: no inputBuffer but input channels");
     }
-    else
-    {        /* PD could not keep up; generate zeros */
-        if (pa_started)
-            pa_dio_error = 1;
-        if (outputBuffer)
-        {
-            for (ch = 0; ch < pa_outchans; ch++)
-            {
-                unsigned long frame;
-                fbuf = ((float *)outputBuffer) + ch;
-                for (frame = 0; frame < nframes; frame++, fbuf += pa_outchans)
-                    *fbuf = 0;
-            }
-        }
-    }
 #ifdef THREADSIGNAL
-    pthread_mutex_lock(&pa_mutex);
-    pthread_cond_signal(&pa_sem);
-    pthread_mutex_unlock(&pa_mutex);
+    sys_semaphore_post(pa_sem);
 #endif
     return 0;
 }
-#endif /* FAKEBLOCKING */
 
-PaError pa_open_callback(double sampleRate, int inchannels, int outchannels,
-    int framesperbuf, int nbuffers, int indeviceno, int outdeviceno, PaStreamCallback *callbackfn)
+PaError pa_open_callback(double samplerate, int inchannels, int outchannels,
+    int framesperbuf, int indeviceno, int outdeviceno, PaStreamCallback *callbackfn)
 {
-    long   bytesPerSample;
+    long bytesPerSample;
     PaError err;
     PaStreamParameters instreamparams, outstreamparams;
     PaStreamParameters*p_instreamparams=0, *p_outstreamparams=0;
@@ -254,23 +228,17 @@ PaError pa_open_callback(double sampleRate, int inchannels, int outchannels,
     instreamparams.channelCount = inchannels;
     instreamparams.sampleFormat = paFloat32;
     instreamparams.hostApiSpecificStreamInfo = 0;
+    instreamparams.suggestedLatency = 0;
+    if (inchannels > 0 && indeviceno >= 0)
+        p_instreamparams = &instreamparams;
 
     outstreamparams.device = outdeviceno;
     outstreamparams.channelCount = outchannels;
     outstreamparams.sampleFormat = paFloat32;
     outstreamparams.hostApiSpecificStreamInfo = 0;
-
-#ifdef FAKEBLOCKING
-    instreamparams.suggestedLatency = outstreamparams.suggestedLatency = 0;
-#else
-    instreamparams.suggestedLatency = outstreamparams.suggestedLatency =
-        nbuffers*framesperbuf/sampleRate;
-#endif /* FAKEBLOCKING */
-
-    if( inchannels>0 && indeviceno >= 0)
-        p_instreamparams=&instreamparams;
-    if( outchannels>0 && outdeviceno >= 0)
-        p_outstreamparams=&outstreamparams;
+    outstreamparams.suggestedLatency = 0;
+    if (outchannels > 0 && outdeviceno >= 0)
+        p_outstreamparams = &outstreamparams;
 
 #ifdef _WIN32
         /* set WASAPI options */
@@ -295,71 +263,60 @@ PaError pa_open_callback(double sampleRate, int inchannels, int outchannels,
     }
 #endif
 
-    err=Pa_IsFormatSupported(p_instreamparams, p_outstreamparams, sampleRate);
-
-    if (paFormatIsSupported != err)
+    err = Pa_IsFormatSupported(p_instreamparams, p_outstreamparams, samplerate);
+    if (err != paFormatIsSupported)
     {
         /* check whether we have to change the numbers of channel and/or samplerate */
         const PaDeviceInfo* info = 0;
-        double inRate=0, outRate=0;
+        double inrate = 0, outrate = 0;
 
-        if (inchannels>0)
+        if (inchannels > 0)
         {
-            if (NULL != (info = Pa_GetDeviceInfo( instreamparams.device )))
+            if ((info = Pa_GetDeviceInfo(instreamparams.device)) != NULL)
             {
-              inRate=info->defaultSampleRate;
-
-              if(info->maxInputChannels<inchannels)
-                instreamparams.channelCount=info->maxInputChannels;
+                inrate = info->defaultSampleRate;
+                if (info->maxInputChannels < inchannels)
+                    instreamparams.channelCount = info->maxInputChannels;
             }
         }
 
-        if (outchannels>0)
+        if (outchannels > 0)
         {
-            if (NULL != (info = Pa_GetDeviceInfo( outstreamparams.device )))
+            if ((info = Pa_GetDeviceInfo(outstreamparams.device)) != NULL)
             {
-              outRate=info->defaultSampleRate;
-
-              if(info->maxOutputChannels<outchannels)
-                outstreamparams.channelCount=info->maxOutputChannels;
+                outrate = info->defaultSampleRate;
+                if (info->maxOutputChannels < outchannels)
+                    outstreamparams.channelCount = info->maxOutputChannels;
             }
         }
 
         if (err == paInvalidSampleRate)
         {
-            double oldrate = sampleRate;
-            sampleRate = outRate > 0 ? outRate : inRate;
+            double oldrate = samplerate;
+            samplerate = outrate > 0 ? outrate : inrate;
             logpost(0, PD_NORMAL,
                 "warning: requested samplerate %d not supported, using %d.",
-                    (int)oldrate, (int)sampleRate);
+                    (int)oldrate, (int)samplerate);
         }
 
-        err=Pa_IsFormatSupported(p_instreamparams, p_outstreamparams,
-            sampleRate);
-        if (paFormatIsSupported != err)
-        goto error;
+        err = Pa_IsFormatSupported(p_instreamparams, p_outstreamparams, samplerate);
+        if (err != paFormatIsSupported)
+            goto error;
     }
-    err = Pa_OpenStream(
-              &pa_stream,
-              p_instreamparams,
-              p_outstreamparams,
-              sampleRate,
-              framesperbuf,
-              paNoFlag,      /* portaudio will clip for us */
-              callbackfn,
-              0);
+    err = Pa_OpenStream(&pa_stream, p_instreamparams, p_outstreamparams,
+        samplerate, framesperbuf, paNoFlag, callbackfn, 0);
     if (err != paNoError)
         goto error;
 
     err = Pa_StartStream(pa_stream);
     if (err != paNoError)
     {
-        post("error opening failed; closing audio stream: %s",
+        post("could not start stream; closing audio: %s",
             Pa_GetErrorText(err));
         pa_close_audio();
         goto error;
     }
-    STUFF->st_dacsr=sampleRate;
+    STUFF->st_dacsr = samplerate;
     return paNoError;
 error:
     pa_stream = NULL;
@@ -399,6 +356,12 @@ int pa_open_audio(int inchans, int outchans, int rate, t_sample *soundin,
                 devno++;
             }
         }
+        if (pa_indev == -1)
+        {
+            inchans = 0;
+            pd_error(0, "audio input device number (%d) out of range",
+                indeviceno);
+        }
     }
 
     if (outchans > 0)
@@ -419,46 +382,44 @@ int pa_open_audio(int inchans, int outchans, int rate, t_sample *soundin,
                 devno++;
             }
         }
+        if (pa_outdev == -1)
+        {
+            outchans = 0;
+            pd_error(0, "audio output device number (%d) out of range",
+                outdeviceno);
+        }
     }
 
-    if (inchans > 0 && pa_indev == -1)
-        inchans = 0;
-    if (outchans > 0 && pa_outdev == -1)
-        outchans = 0;
-
     logpost(NULL, PD_VERBOSE, "input device %d, channels %d", pa_indev, inchans);
     logpost(NULL, PD_VERBOSE, "output device %d, channels %d", pa_outdev, outchans);
     logpost(NULL, PD_VERBOSE, "framesperbuf %d, nbufs %d", framesperbuf, nbuffers);
     logpost(NULL, PD_VERBOSE, "rate %d", rate);
 
+    if (!inchans && !outchans)
+        return (1);
+
     pa_inchans = STUFF->st_inchannels = inchans;
     pa_outchans = STUFF->st_outchannels = outchans;
     pa_soundin = soundin;
     pa_soundout = soundout;
 
-#ifdef FAKEBLOCKING
     if (pa_inbuf)
         free((char *)pa_inbuf), pa_inbuf = 0;
     if (pa_outbuf)
         free((char *)pa_outbuf), pa_outbuf = 0;
 #ifdef THREADSIGNAL
-    pthread_mutex_init(&pa_mutex, 0);
-    pthread_cond_init(&pa_sem, 0);
+    pa_sem = sys_semaphore_create();
 #endif
-#endif
-
-    if (! inchans && !outchans)
-        return (0);
+    pa_lastdactime = 0;
 
     if (callbackfn)
     {
         pa_callback = callbackfn;
         err = pa_open_callback(rate, inchans, outchans,
-            framesperbuf, nbuffers, pa_indev, pa_outdev, pa_lowlevel_callback);
+            framesperbuf, pa_indev, pa_outdev, pa_lowlevel_callback);
     }
     else
     {
-#ifdef FAKEBLOCKING
         if (pa_inchans)
         {
             pa_inbuf = malloc(nbuffers*framesperbuf*pa_inchans*sizeof(float));
@@ -473,15 +434,11 @@ int pa_open_audio(int inchans, int outchans, int rate, t_sample *soundin,
                 nbuffers*framesperbuf*pa_outchans*sizeof(float), pa_outbuf, 0);
         }
         err = pa_open_callback(rate, inchans, outchans,
-            framesperbuf, nbuffers, pa_indev, pa_outdev, pa_fifo_callback);
-#else
-        err = pa_open_callback(rate, inchans, outchans,
-            framesperbuf, nbuffers, pa_indev, pa_outdev, 0);
-#endif
+            framesperbuf, pa_indev, pa_outdev, pa_fifo_callback);
     }
     pa_started = 0;
-    pa_nbuffers = nbuffers;
-    if ( err != paNoError )
+    pa_dio_error = 0;
+    if (err != paNoError)
     {
         pd_error(0, "error opening audio: %s", Pa_GetErrorText(err));
         /* Pa_Terminate(); */
@@ -499,87 +456,92 @@ void pa_close_audio(void)
         Pa_CloseStream(pa_stream);
     }
     pa_stream = 0;
-#ifdef FAKEBLOCKING
     if (pa_inbuf)
         free((char *)pa_inbuf), pa_inbuf = 0;
     if (pa_outbuf)
         free((char *)pa_outbuf), pa_outbuf = 0;
 #ifdef THREADSIGNAL
-    pthread_mutex_destroy(&pa_mutex);
-    pthread_cond_destroy(&pa_sem);
-#endif
+    if (pa_sem)
+    {
+        sys_semaphore_destroy(pa_sem);
+        pa_sem = 0;
+    }
 #endif
 }
 
+void sys_do_close_audio(void);
+void sys_do_reopen_audio(void);
+
+int pa_reopen_audio(void)
+{
+    int success;
+        /* NB: unfortunately, Pa_IsStreamActive() always returns true,
+           even if the audio device has been disconnected... */
+    sys_do_close_audio();
+#ifdef __APPLE__
+        /* TODO: the portaudio coreaudio implementation doesn't handle
+           re-connection, so suggest restarting */
+    pd_error(0, "audio device not responding - closing audio");
+    pd_error(0, "you may need to save and restart pd");
+    return 0;
+#else
+        /* portaudio seems to handle this better on windows and linux */
+    pd_error(0, "trying to reopen audio device");
+
+    sys_do_reopen_audio(); /* try to reopen it */
+    success = audio_isopen();
+
+    if (success)
+        pd_error(0, "successfully reopened audio device");
+    else
+        pd_error(0, "audio device not responding - closing audio.\n"
+                    "please try to reconnect and reselect it in the settings (or toggle DSP)");
+#endif
+    return success;
+}
+
+int sched_idletask(void);
+
 int pa_send_dacs(void)
 {
     t_sample *fp;
     float *fp2, *fp3;
     float *conversionbuf;
     int j, k;
-    int rtnval =  SENDDACS_YES;
-    int locked = 0;
-    double timebefore;
-#ifdef FAKEBLOCKING
-#ifdef THREADSIGNAL
-    struct timespec ts;
-    double timeout;
-#ifdef _WIN32
-    struct __timeb64 tb;
-    _ftime64(&tb);
-    timeout = (double)tb.time + tb.millitm * 0.001;
-#else
-    struct timeval now;
-    gettimeofday(&now, 0);
-    timeout = (double)now.tv_sec + (1./1000000.) * now.tv_usec;
-#endif
-    timeout += THREADSIGNAL_TIMEOUT * 0.001;
-    ts.tv_sec = (long long)timeout;
-    ts.tv_nsec = (timeout - (double)ts.tv_sec) * 1e9;
-#else
-#endif /* THREADSIGNAL */
-#endif /* FAKEBLOCKING */
-    timebefore = sys_getrealtime();
-
-    if ((!STUFF->st_inchannels && !STUFF->st_outchannels) || !pa_stream)
+    int retval = SENDDACS_YES;
+    double timeref = sys_getrealtime();
+        /* this shouldn't really happen... */
+    if (!pa_stream || (!STUFF->st_inchannels && !STUFF->st_outchannels))
         return (SENDDACS_NO);
+
     conversionbuf = (float *)alloca((STUFF->st_inchannels > STUFF->st_outchannels?
         STUFF->st_inchannels:STUFF->st_outchannels) * DEFDACBLKSIZE * sizeof(float));
 
-#ifdef FAKEBLOCKING
-    if (!STUFF->st_inchannels)    /* if no input channels sync on output */
+    while (
+        (sys_ringbuf_getreadavailable(&pa_inring) <
+            (long)(STUFF->st_inchannels * DEFDACBLKSIZE*sizeof(t_sample))) ||
+        (sys_ringbuf_getwriteavailable(&pa_outring) <
+            (long)(STUFF->st_outchannels * DEFDACBLKSIZE*sizeof(t_sample))))
     {
 #ifdef THREADSIGNAL
-        pthread_mutex_lock(&pa_mutex);
-#endif
-        while (sys_ringbuf_getwriteavailable(&pa_outring) <
-            (long)(STUFF->st_outchannels * DEFDACBLKSIZE * sizeof(float)))
+        if (sched_idletask())
+            continue;
+            /* only go to sleep if there is nothing else to do. */
+        if (!sys_semaphore_waitfor(pa_sem, POLL_TIMEOUT))
         {
-            rtnval = SENDDACS_SLEPT;
-#ifdef THREADSIGNAL
-            if (pthread_cond_timedwait(&pa_sem, &pa_mutex, &ts) == ETIMEDOUT)
-            {
-                locked = 1;
-                break;
-            }
-#else
-            if (Pa_IsStreamActive(&pa_stream) < 0 &&
-                (sys_getrealtime() - timebefore) > (POLL_TIMEOUT * 0.001))
-            {
-                locked = 1;
-                break;
-            }
-            sys_microsleep();
-            if (!pa_stream)     /* sys_microsleep() may have closed device */
-                return SENDDACS_NO;
-#endif /* THREADSIGNAL */
+            pa_reopen_audio();
+            return SENDDACS_NO;
         }
-#ifdef THREADSIGNAL
-        pthread_mutex_unlock(&pa_mutex);
+        retval = SENDDACS_SLEPT;
+#else
+        if ((timeref - pa_lastdactime) >= POLL_TIMEOUT)
+            pa_reopen_audio();
+        return SENDDACS_NO;
 #endif
     }
+    pa_lastdactime = timeref;
         /* write output */
-    if (STUFF->st_outchannels && !locked)
+    if (STUFF->st_outchannels)
     {
         for (j = 0, fp = STUFF->st_soundout, fp2 = conversionbuf;
             j < STUFF->st_outchannels; j++, fp2++)
@@ -589,38 +551,8 @@ int pa_send_dacs(void)
         sys_ringbuf_write(&pa_outring, conversionbuf,
             STUFF->st_outchannels*(DEFDACBLKSIZE*sizeof(float)), pa_outbuf);
     }
-    if (STUFF->st_inchannels)    /* if there is input sync on it */
-    {
-#ifdef THREADSIGNAL
-        pthread_mutex_lock(&pa_mutex);
-#endif
-        while (sys_ringbuf_getreadavailable(&pa_inring) <
-            (long)(STUFF->st_inchannels * DEFDACBLKSIZE * sizeof(float)))
-        {
-            rtnval = SENDDACS_SLEPT;
-#ifdef THREADSIGNAL
-            if (pthread_cond_timedwait(&pa_sem, &pa_mutex, &ts) == ETIMEDOUT)
-            {
-                locked = 1;
-                break;
-            }
-#else
-            if (Pa_IsStreamActive(&pa_stream) < 0 &&
-                (sys_getrealtime() - timebefore) > (POLL_TIMEOUT * 0.001))
-            {
-                locked = 1;
-                break;
-            }
-            sys_microsleep();
-            if (!pa_stream)     /* sys_microsleep() may have closed device */
-                return SENDDACS_NO;
-#endif /* THREADSIGNAL */
-        }
-#ifdef THREADSIGNAL
-        pthread_mutex_unlock(&pa_mutex);
-#endif
-    }
-    if (STUFF->st_inchannels && !locked)
+        /* read input */
+    if (STUFF->st_inchannels)
     {
         sys_ringbuf_read(&pa_inring, conversionbuf,
             STUFF->st_inchannels*(DEFDACBLKSIZE*sizeof(float)), pa_inbuf);
@@ -631,72 +563,17 @@ int pa_send_dacs(void)
                         *fp = *fp3;
     }
 
-#else /* FAKEBLOCKING */
-        /* write output */
-    if (STUFF->st_outchannels)
+    if (pa_dio_error)
     {
-        if (!pa_started)
-        {
-            memset(conversionbuf, 0,
-                STUFF->st_outchannels * DEFDACBLKSIZE * sizeof(float));
-            for (j = 0; j < pa_nbuffers-1; j++)
-                Pa_WriteStream(pa_stream, conversionbuf, DEFDACBLKSIZE);
-        }
-        for (j = 0, fp = STUFF->st_soundout, fp2 = conversionbuf;
-            j < STUFF->st_outchannels; j++, fp2++)
-                for (k = 0, fp3 = fp2; k < DEFDACBLKSIZE;
-                    k++, fp++, fp3 += STUFF->st_outchannels)
-                        *fp3 = *fp;
-        if (Pa_WriteStream(pa_stream, conversionbuf, DEFDACBLKSIZE) != paNoError)
-            if (Pa_IsStreamActive(&pa_stream) < 0)
-                locked = 1;
+        sys_log_error(ERR_RESYNC);
+        pa_dio_error = 0;
     }
-
-    if (STUFF->st_inchannels)
-    {
-        if (Pa_ReadStream(pa_stream, conversionbuf, DEFDACBLKSIZE) != paNoError)
-            if (Pa_IsStreamActive(&pa_stream) < 0)
-                locked = 1;
-        for (j = 0, fp = STUFF->st_soundin, fp2 = conversionbuf;
-            j < STUFF->st_inchannels; j++, fp2++)
-                for (k = 0, fp3 = fp2; k < DEFDACBLKSIZE;
-                    k++, fp++, fp3 += STUFF->st_inchannels)
-                        *fp = *fp3;
-    }
-    if (sys_getrealtime() - timebefore > 0.002)
-    {
-        rtnval = SENDDACS_SLEPT;
-    }
-#endif /* FAKEBLOCKING */
     pa_started = 1;
 
     memset(STUFF->st_soundout, 0,
         DEFDACBLKSIZE*sizeof(t_sample)*STUFF->st_outchannels);
-    if (locked)
-    {
-        PaError err = Pa_IsStreamActive(&pa_stream);
-        pd_error(0, "error %d: %s", err, Pa_GetErrorText(err));
-        sys_close_audio();
-        #ifdef __APPLE__
-            /* TODO: the portaudio coreaudio implementation doesn't handle
-               re-connection, so suggest restarting */
-            pd_error(0, "audio device not responding - closing audio");
-            pd_error(0, "you may need to save and restart pd");
-        #else
-            /* portaudio seems to handle this better on windows and linux */
-            pd_error(0, "trying to reopen audio device");
-            sys_reopen_audio(); /* try to reopen it */
-            if (audio_isopen())
-                pd_error(0, "successfully reopened audio device");
-            else
-            {
-                pd_error(0, "audio device not responding - closing audio");
-                pd_error(0, "reconnect and reselect it in the settings (or toggle DSP)");
-            }
-        #endif
-        return SENDDACS_NO;
-    }
-    else return (rtnval);
+
+    return retval;
 }
 
 static char*pdi2devname(const PaDeviceInfo*pdi, char*buf, size_t bufsize) {
@@ -709,10 +586,10 @@ static char*pdi2devname(const PaDeviceInfo*pdi, char*buf, size_t bufsize) {
 
     api = Pa_GetHostApiInfo(pdi->hostApi);
     if(api)
-        snprintf(utf8device, MAXPDSTRING, "%s: %s",
+        pd_snprintf(utf8device, MAXPDSTRING, "%s: %s",
             api->name, pdi->name);
     else
-        snprintf(utf8device, MAXPDSTRING, "%s",
+        pd_snprintf(utf8device, MAXPDSTRING, "%s",
             pdi->name);
 
     u8_nativetoutf8(buf, bufsize, utf8device, MAXPDSTRING);
@@ -738,13 +615,13 @@ void pa_getdevs(char *indevlist, int *nindevs,
         if (pdi->maxInputChannels > 0 && nin < maxndev)
         {
                 /* LATER figure out how to get API name correctly */
-            snprintf(indevlist + nin * devdescsize, devdescsize,
+            pd_snprintf(indevlist + nin * devdescsize, devdescsize,
                 "%s", devname);
             nin++;
         }
         if (pdi->maxOutputChannels > 0 && nout < maxndev)
         {
-            snprintf(outdevlist + nout * devdescsize, devdescsize,
+            pd_snprintf(outdevlist + nout * devdescsize, devdescsize,
                 "%s", devname);
             nout++;
         }
