@@ -7,6 +7,7 @@ away before the panel does... */
 
 #include "m_pd.h"
 #include "g_canvas.h"
+#include "s_stuff.h"
 #include <stdio.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
@@ -16,6 +17,17 @@ away before the panel does... */
 
 /* --------------------- graphics responder  ---------------- */
 
+/* gfxstub_* are deprecated for externals and shouldn't be used directly within Pd.
+   however, the we do use them for implementing the high-level
+   communication pdgui_stub_*(), so we do not want the compiler to shout out loud.
+ */
+#ifdef __GNUC__
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#elif defined _MSC_VER
+#pragma warning( disable : 4996 )
+#endif
+
+
 /* make one of these if you want to put up a dialog window but want to be
 protected from getting deleted and then having the dialog call you back.  In
 this design the calling object doesn't have to keep the address of the dialog
@@ -240,6 +252,7 @@ static t_class *openpanel_class;
 typedef struct _openpanel
 {
     t_object x_obj;
+    t_canvas *x_canvas;
     t_symbol *x_s;
     int x_mode; /* 0: file, 1: folder, 2: multiple files */
 } t_openpanel;
@@ -252,6 +265,7 @@ static void *openpanel_new(t_floatarg mode)
     x->x_mode = (mode < 0 || mode > 2) ? 0 : mode;
     sprintf(buf, "d%lx", (t_int)x);
     x->x_s = gensym(buf);
+    x->x_canvas = canvas_getcurrent();
     pd_bind(&x->x_obj.ob_pd, x->x_s);
     outlet_new(&x->x_obj, &s_symbol);
     return (x);
@@ -260,8 +274,8 @@ static void *openpanel_new(t_floatarg mode)
 static void openpanel_symbol(t_openpanel *x, t_symbol *s)
 {
     const char *path = (s && s->s_name) ? s->s_name : "\"\"";
-    pdgui_vmess("pdtk_openpanel", "ssi",
-        x->x_s->s_name, path, x->x_mode);
+    pdgui_vmess("pdtk_openpanel", "ssic",
+        x->x_s->s_name, path, x->x_mode, glist_getcanvas(x->x_canvas));
 }
 
 static void openpanel_bang(t_openpanel *x)
@@ -324,8 +338,8 @@ static void *savepanel_new(void)
 static void savepanel_symbol(t_savepanel *x, t_symbol *s)
 {
     const char *path = (s && s->s_name) ? s->s_name : "\"\"";
-    pdgui_vmess("pdtk_savepanel", "ss",
-        x->x_s->s_name, path);
+    pdgui_vmess("pdtk_savepanel", "ssc",
+        x->x_s->s_name, path, glist_getcanvas(x->x_canvas));
 }
 
 static void savepanel_bang(t_savepanel *x)
@@ -489,7 +503,7 @@ static void pdcontrol_dir(t_pdcontrol *x, t_symbol *s, t_floatarg f)
     if (*s->s_name)
     {
         char buf[MAXPDSTRING];
-        snprintf(buf, MAXPDSTRING, "%s/%s",
+        pd_snprintf(buf, MAXPDSTRING, "%s/%s",
             canvas_getdir(c)->s_name, s->s_name);
         buf[MAXPDSTRING-1] = 0;
         outlet_symbol(x->x_outlet, gensym(buf));
