@@ -12,13 +12,12 @@ if { [catch {wm withdraw .} fid] } { exit 2 }
 
 # This is mainly for OSX as older versions only
 # have 8.4 while newer versions have 8.5.
-if { [catch {package provide Tcl 8.5}] } {
+if { [catch {package require Tcl 8.5 9}] } {
     # Tcl 8.5 not available
     package require Tcl 8.4
     package require Tk
 } else {
-    # Tcl 8.5 is available
-    package require Tcl 8.5
+    # Tcl >=8.5,>=9 is available
     package require Tk
 
     # replace Tk widgets with Ttk widgets on 8.5
@@ -532,8 +531,6 @@ proc pdtk_pd_startup {major minor bugfix test
     set_base_font $sys_font $sys_fontweight
     set ::font_measured [fit_font_into_metrics $::font_family $::font_weight $::font_metrics]
     set ::font_zoom2_measured [fit_font_into_metrics $::font_family $::font_weight $::font_zoom2_metrics]
-    pdsend "pd init [enquote_path [pwd]] $oldtclversion \
-        $::font_measured $::font_zoom2_measured"
     ::pd_bindings::class_bindings
     ::pd_bindings::global_bindings
     ::pd_menus::create_menubar
@@ -542,6 +539,8 @@ proc pdtk_pd_startup {major minor bugfix test
     ::pd_menus::configure_for_pdwindow
     ::pdwindow::create_window_finalize
     load_startup_plugins
+    pdsend "pd init [enquote_path [pwd]] $oldtclversion \
+        $::font_measured $::font_zoom2_measured"
     open_filestoopen
     set ::done_init 1
 }
@@ -574,12 +573,13 @@ proc pdtk_yesnodialog {mytoplevel message default} {
 ## pdtk_check .pdwindow "Hello world!" "pd dsp 1" no
 proc pdtk_check {mytoplevel message reply_to_pd default} {
     # example: 'pdtk_check . [list "Switch compatibility to %s?" $compat] [list pd compatibility $compat ] no'
+
     if {[lindex $message 0] == [lindex [lindex $message 0] 0]} {
         set message [ list $message ]
     }
 
     if {[ catch {
-              set msg [format [_ [ lindex $message 0 ] ] {*}[lrange $message 1 end] ]
+              set msg [_ {*}$message]
           } ]} {
            set msg [_ $message]
        }
