@@ -12,17 +12,17 @@ namespace eval ::pd_guiprefs:: {
     namespace export write_recentfiles
     namespace export update_recentfiles
     namespace export write_loglevel
+
+    # preference keys
+    variable recentfiles_key ""
+    variable loglevel_key "loglevel"
+
+    # platform specific
+    variable domain org.puredata.pd.pd-gui
+    variable configdir ""
+    variable recentfiles_is_array false
 }
 
-# preference keys
-set ::pd_guiprefs::recentfiles_key ""
-set ::pd_guiprefs::loglevel_key "loglevel"
-
-# platform specific
-set ::pd_guiprefs::domain ""
-set ::pd_guiprefs::configdir ""
-set ::pd_guiprefs::recentfiles_is_array false
-
 #################################################################
 # preferences storage locations
 #
@@ -65,11 +65,13 @@ set ::pd_guiprefs::recentfiles_is_array false
 # init preferences
 #
 proc ::pd_guiprefs::init {} {
-    set ::pd_guiprefs::domain org.puredata.pd.pd-gui
-
     switch -- $::platform {
         "Darwin" {
             set backend "plist"
+            ## on macOS the domain should be the same as the bundle ID
+            if {[info exists ::env(__CFBundleIdentifier)] && [string trim $::env(__CFBundleIdentifier)] != {}} {
+                set ::pd_guiprefs::domain $::env(__CFBundleIdentifier)
+            }
         }
         "W32" {
             set backend "registry"
@@ -94,14 +96,14 @@ proc ::pd_guiprefs::init {} {
             # ------------------------------------------------------------------------------
             # macOS: read a plist file using the 'defaults' command
             #
-            proc ::pd_guiprefs::get_config {domain key {arr false}} {
+            proc ::pd_guiprefs::get_config {domain key {islist false}} {
                 if {![catch {exec defaults read $domain $key} conf]} {
-                    if {$arr} {
+                    if {$islist} {
                         set conf [plist_array_to_tcl_list $conf]
                     }
                 } else {
                     # value not found, so set empty value
-                    if {$arr} {
+                    if {$islist} {
                         # initialize w/ empty array for NSRecentDocuments, etc
                         exec defaults write $domain $key -array
                         set conf {}
@@ -115,10 +117,10 @@ proc ::pd_guiprefs::init {} {
             }
             # ------------------------------------------------------------------------------
             # macOS: write configs to plist file using the 'defaults' command
-            # if $arr is true, we write an array
+            # if $islist is true, we write a list
             #
-            proc ::pd_guiprefs::write_config {data domain key {arr false}} {
-                if {$arr} {
+            proc ::pd_guiprefs::write_config {data domain key {islist false}} {
+                if {$islist} {
                     # FIXME empty and write again so we don't lose the order
                     if {[catch {exec defaults write $domain $key -array} errorMsg]} {
                         puts "write_config $key: $errorMsg\n"
@@ -130,8 +132,7 @@ proc ::pd_guiprefs::init {} {
                         }
                     }
                 } else {
-                    set escaped [escape_for_plist $data]
-                    if {[catch {exec defaults write $domain $key $escaped} errorMsg]} {
+                    if {[catch {exec defaults write $domain $key -string $data} errorMsg]} {
                         puts "write_config $key: $errorMsg\n"
                     }
                 }
@@ -179,7 +180,7 @@ proc ::pd_guiprefs::init {} {
             # ------------------------------------------------------------------------------
             # w32: read in the registry
             #
-            proc ::pd_guiprefs::get_config {domain key {arr false}} {
+            proc ::pd_guiprefs::get_config {domain key {islist false}} {
                 package require registry
                 set domain [join [list ${::pd_guiprefs::registrypath} ${domain}] \\]
                 if {![catch {registry get ${domain} $key} conf]} {
@@ -189,13 +190,13 @@ proc ::pd_guiprefs::init {} {
             }
             # ------------------------------------------------------------------------------
             # w32: write configs to registry
-            # if $arr is true, we write an array
+            # if $islist is true, we write a list
             #
-            proc ::pd_guiprefs::write_config {data domain key {arr false}} {
+            proc ::pd_guiprefs::write_config {data domain key {islist false}} {
                 package require registry
                 # FIXME: ugly
                 set domain [join [list ${::pd_guiprefs::registrypath} ${domain}] \\]
-                if {$arr} {
+                if {$islist} {
                     if {[catch {registry set ${domain} $key $data multi_sz} errorMsg]} {
                         ::pdwindow::error "write_config $data $key: $errorMsg\n"
                     }
@@ -236,15 +237,15 @@ proc ::pd_guiprefs::init {} {
             # ------------------------------------------------------------------------------
             # linux: read a config file and return its lines split.
             #
-            proc ::pd_guiprefs::get_config {domain key {arr false}} {
-                return [::pd_guiprefs::get_config_file $domain $key $arr]
+            proc ::pd_guiprefs::get_config {domain key {islist false}} {
+                return [::pd_guiprefs::get_config_file $domain $key $islist]
             }
             # ------------------------------------------------------------------------------
             # linux: write configs to USER_APP_CONFIG_DIR
-            # $arr is true if the data needs to be written in an array
+            # $islist is true if the data needs to be written in a list
             #
-            proc ::pd_guiprefs::write_config {data domain key {arr false}} {
-                return [::pd_guiprefs::write_config_file $data $domain $key $arr]
+            proc ::pd_guiprefs::write_config {data domain key {islist false}} {
+                return [::pd_guiprefs::write_config_file $data $domain $key $islist]
             }
             # ------------------------------------------------------------------------------
             # linux: delete config from registry (if $key is empty, delete the entire $domain)
@@ -291,7 +292,7 @@ proc ::pd_guiprefs::init {} {
 # ------------------------------------------------------------------------------
 # read a config file and return its lines split.
 #
-proc ::pd_guiprefs::get_config_file {domain key {arr false}} {
+proc ::pd_guiprefs::get_config_file {domain key {islist false}} {
     set filename [file join ${::pd_guiprefs::configdir} ${domain} ${key}.conf]
     set conf {}
     if {
@@ -299,21 +300,21 @@ proc ::pd_guiprefs::get_config_file {domain key {arr false}} {
         && [file readable $filename]
     } {
         set fl [open $filename r]
-        while {[gets $fl line] >= 0} {
-            lappend conf $line
-        }
+        set conf [::read -nonewline $fl]
         close $fl
     }
     return $conf
 }
 # ------------------------------------------------------------------------------
 # write configs to USER_APP_CONFIG_DIR
-# $arr is true if the data needs to be written in an array
+# $islist is true if the data needs to be written in a list
 #
-proc ::pd_guiprefs::write_config_file {data domain key {arr false}} {
+proc ::pd_guiprefs::write_config_file {data domain key {islist false}} {
     ::pd_guiprefs::prepare_domain ${domain}
-    # right now I (yvan) assume that data are just \n separated, i.e. no keys
-    set data [join $data "\n"]
+    ## originally, yvan assumed that data are just \n separated, i.e. no keys
+    #set data [join $data "\n"]
+    # this however breaks if we do have data that contains \n.
+    # much better to just let Tcl handle the serialization
     set filename [file join ${::pd_guiprefs::configdir} ${domain} ${key}.conf]
     if {[catch {set fl [open $filename w]} errorMsg]} {
         ::pdwindow::error "write_config $data $key: $errorMsg\n"
@@ -328,10 +329,10 @@ proc ::pd_guiprefs::write_config_file {data domain key {arr false}} {
 #################################################################
 
 ## these are stubs that will be overwritten in ::pd_guiprefs::init()
-proc ::pd_guiprefs::write_config {data domain key {arr false}} {
+proc ::pd_guiprefs::write_config {data domain key {islist false}} {
     ::pdwindow::error "::pd_guiprefs::write_config not implemented for $::platform\n"
 }
-proc ::pd_guiprefs::get_config {domain key {arr false}} {
+proc ::pd_guiprefs::get_config {domain key {islist false}} {
     ::pdwindow::error "::pd_guiprefs::get_config not implemented for $::platform\n"
 }
 proc ::pd_guiprefs::delete_config {{domain {}} {key {}}} {
@@ -340,14 +341,14 @@ proc ::pd_guiprefs::delete_config {{domain {}} {key {}}} {
 }
 
 # simple API (with a default domain)
-proc ::pd_guiprefs::write {key data {arr false} {domain {}}} {
+proc ::pd_guiprefs::write {key data {islist false} {domain {}}} {
     if {"" eq $domain} { set domain ${::pd_guiprefs::domain} }
-    set result [::pd_guiprefs::write_config $data $domain $key $arr]
+    set result [::pd_guiprefs::write_config $data $domain $key $islist]
     return $result
 }
-proc ::pd_guiprefs::read {key {arr false} {domain {}}} {
+proc ::pd_guiprefs::read {key {islist false} {domain {}}} {
     if {"" eq $domain} { set domain ${::pd_guiprefs::domain} }
-    set result [::pd_guiprefs::get_config $domain $key $arr]
+    set result [::pd_guiprefs::get_config $domain $key $islist]
     return $result
 }
 
@@ -384,6 +385,8 @@ proc ::pd_guiprefs::prepare_configdir {domain} {
         }
     }
 
+    catch { set confdir [file tildeexpand $confdir] }
+
     set ::pd_guiprefs::configdir $confdir
     set ::pd_guiprefs::domain $domain
 
@@ -402,7 +405,7 @@ proc ::pd_guiprefs::prepare_domain {{domain {}}} {
         set absconfdir ${::pd_guiprefs::configdir}
         catch { set absconfdir [file normalize ${::pd_guiprefs::configdir} ] }
 
-        ::pdwindow::error [format [_ "Couldn't create preferences \"%1\$s\" in %2\$s" ] $domain $absconfdir]
+        ::pdwindow::error [_ "Couldn't create preferences \"%1\$s\" in %2\$s" $domain $absconfdir]
         ::pdwindow::error "\n"
     }
     return $domain
@@ -411,9 +414,9 @@ proc ::pd_guiprefs::prepare_domain {{domain {}}} {
 # ------------------------------------------------------------------------------
 # convenience proc to init prefs value, returns default if not found
 #
-proc ::pd_guiprefs::init_config {domain key {default ""} {arr false}} {
+proc ::pd_guiprefs::init_config {domain key {default ""} {islist false}} {
     set conf ""
-    catch {set conf [::pd_guiprefs::get_config $domain $key $arr]}
+    catch {set conf [::pd_guiprefs::get_config $domain $key $islist]}
     if {$conf eq ""} {set conf $default}
     return $conf
 }
